//-----------------------------------------------------------------------------
// Game1.cs
//
// Kyoung Shin Park @ Dankook University (5/6/2012)
//-----------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using XNAGeometryObject;

namespace XNA4AlphaTexture
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // model
        Model[] myModel = new Model[2];
        Vector3 modelPosition = Vector3.Zero;
        Vector3 modelPosition2 = Vector3.One;
        // box
        Box[] box = new Box[2];
        Texture2D[] boxTexture = new Texture2D[2];
        // quad
        Quad[] quad = new Quad[2];
        Texture2D[] quadTexture = new Texture2D[2];

        // effect
        BasicEffect boxEffect;
        BasicEffect quadEffect;
        Matrix projection;
        Matrix view;
        Matrix world;

        // input
        GamePadState currentGamePadState;
        GamePadState prevGamePadState;
        KeyboardState currentKeyboardState;
        KeyboardState prevKeyboardState;
        float angle = 0.0f;
        float zoom = 1.0f;
        bool animationMode = false;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            base.Initialize();

            box[0] = new Box(1);
            box[1] = new Box(1);

            quad[0] = new Quad(new Vector3(-1, 0, 0), Vector3.Backward, Vector3.Up, 1.5f, 1.5f);
            quad[1] = new Quad(new Vector3(1, 0, 0), Vector3.Backward, Vector3.Up, 1.5f, 1.5f);

            InitializeTransform();
            InitializeEffect();
        }

        /// <summary>
        /// Initializes the transforms used by the game.
        /// </summary>
        private void InitializeTransform()
        {
            // world matrix
            world = Matrix.Identity;

            // view matrix
            view = Matrix.CreateLookAt(new Vector3(0, 0, 5.0f), Vector3.Zero, Vector3.Up);

            // projection matrix
            float aspectRatio = (float)GraphicsDevice.Viewport.Width / (float)GraphicsDevice.Viewport.Height;
            projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, aspectRatio, 1.0f, 1000.0f);
        }

        /// <summary>
        /// Initializes the effect (loading, parameter setting, and technique selection)
        /// used by the game.
        /// </summary>
        private void InitializeEffect()
        {
            // Initialize BasicEffect
            boxEffect = new BasicEffect(GraphicsDevice);
            boxEffect.PreferPerPixelLighting = true;

            boxEffect.DiffuseColor = new Vector3(1.0f, 1.0f, 1.0f);    // object material
            boxEffect.SpecularColor = new Vector3(1.0f, 1.0f, 1.0f);
            boxEffect.SpecularPower = 12.0f;
            boxEffect.AmbientLightColor = new Vector3(0.75f, 0.75f, 0.75f);

            boxEffect.DirectionalLight0.Enabled = true;
            boxEffect.DirectionalLight0.DiffuseColor = Vector3.One;
            boxEffect.DirectionalLight0.Direction = Vector3.Normalize(new Vector3(1.0f, 1.0f, 0.0f));
            boxEffect.DirectionalLight0.SpecularColor = Vector3.One;

            boxEffect.DirectionalLight1.Enabled = true;
            boxEffect.DirectionalLight1.DiffuseColor = new Vector3(1.0f, 1.0f, 1.0f);
            boxEffect.DirectionalLight1.Direction = Vector3.Normalize(new Vector3(-1.0f, -1.0f, 1.0f));
            boxEffect.DirectionalLight1.SpecularColor = new Vector3(0.5f, 0.5f, 0.5f);

            boxEffect.LightingEnabled = true;
            boxEffect.TextureEnabled = true;

            boxEffect.World = world;
            boxEffect.View = view;
            boxEffect.Projection = projection;

            quadEffect = new BasicEffect(GraphicsDevice);
            quadEffect.EnableDefaultLighting();
            quadEffect.TextureEnabled = true; 
            quadEffect.World = world;
            quadEffect.View = view;
            quadEffect.Projection = projection;
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            myModel[0] = Content.Load<Model>("Ship");
            myModel[1] = Content.Load<Model>("p1_wedge");

            boxTexture[0] = Content.Load<Texture2D>("crate");
            boxTexture[1] = Content.Load<Texture2D>("xna_logo"); 

            quadTexture[0] = Content.Load<Texture2D>("nice");
            quadTexture[1] = Content.Load<Texture2D>("danc_princess");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            HandleInput();

            base.Update(gameTime);
        }

        /// <summary>
        /// Determines which primitive to draw based on input from the keyboard
        /// or game pad.
        /// </summary>
        protected void HandleInput()
        {
            CheckGamePadInput();
            CheckKeyboardInput();
        }

        private void CheckGamePadInput()
        {
            prevGamePadState = currentGamePadState;
            currentGamePadState = GamePad.GetState(PlayerIndex.One);
        }

        private void CheckKeyboardInput()
        {
            prevKeyboardState = currentKeyboardState;
            currentKeyboardState = Keyboard.GetState(PlayerIndex.One);

            if (currentKeyboardState.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }
            else if (currentKeyboardState.IsKeyDown(Keys.X)
                && !prevKeyboardState.IsKeyDown(Keys.X))
            {
                angle += 5.0f;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Y)
                && !prevKeyboardState.IsKeyDown(Keys.Y))
            {
                zoom -= 0.5f;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Space)
                && !prevKeyboardState.IsKeyDown(Keys.Space))
            {
                animationMode = !animationMode;
            }
        }

        /// <summary>
        /// Renders the 3D model.
        /// </summary>
        private void DrawModel(Matrix world, Matrix view, Matrix projection, Model m, float alpha)
        {
            // save current states
            BlendState bs = GraphicsDevice.BlendState;
            DepthStencilState dss = GraphicsDevice.DepthStencilState;
            // set for transparencies
            GraphicsDevice.BlendState = BlendState.AlphaBlend;
            GraphicsDevice.DepthStencilState = DepthStencilState.Default;
            GraphicsDevice.SamplerStates[0] = SamplerState.LinearWrap;

            Matrix[] transforms = new Matrix[m.Bones.Count];
            m.CopyAbsoluteBoneTransformsTo(transforms);
            foreach (ModelMesh mesh in m.Meshes)
            {
                foreach (BasicEffect effect in mesh.Effects)
                {
                    effect.EnableDefaultLighting();
                    effect.Alpha = alpha;

                    effect.View = view;
                    effect.Projection = projection;
                    effect.World = Matrix.CreateFromYawPitchRoll(MathHelper.ToRadians(90), 0.0f, 0.0f) * Matrix.CreateScale(0.002f, 0.002f, 0.002f) * transforms[mesh.ParentBone.Index] * world;
                }
                mesh.Draw();
            }

            // reset blending states
            GraphicsDevice.BlendState = bs;
            GraphicsDevice.DepthStencilState = dss;
        }
        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            // Copy any parent transforms.
            /*
            // Draw model
            Matrix modelWorld = Matrix.CreateTranslation(modelPosition) * Matrix.CreateTranslation(0, 0, zoom);
            Matrix modelWorld2 = Matrix.CreateTranslation(modelPosition2) * Matrix.CreateTranslation(0, 0, zoom);
            DrawModel(modelWorld, view, projection, myModel[0], 0.5f);
            DrawModel(modelWorld2, view, projection, myModel[1], 1.0f);
            */
            
            GraphicsDevice.BlendState = BlendState.AlphaBlend;
            GraphicsDevice.DepthStencilState = DepthStencilState.None;

            if (animationMode) angle += 0.8f;
            // Draw the box1
            foreach (EffectPass pass in boxEffect.CurrentTechnique.Passes)
            {
                pass.Apply();
                boxEffect.World = Matrix.CreateFromYawPitchRoll(MathHelper.ToRadians(angle), 45.0f, 0.0f) * Matrix.CreateTranslation(0, 0, zoom) * Matrix.CreateTranslation(-1.5f, 0, -1.0f);
                boxEffect.Texture = boxTexture[0];
                boxEffect.Alpha = 0.5f;
                GraphicsDevice.DrawUserPrimitives<VertexPositionNormalTexture>(PrimitiveType.TriangleList,
                    box[0].Vertices, 0, box[0].Vertices.Length / 3);
            }
            // Draw the box2
            foreach (EffectPass pass in boxEffect.CurrentTechnique.Passes)
            {
                pass.Apply();
                boxEffect.World = Matrix.CreateFromYawPitchRoll(MathHelper.ToRadians(angle), 45.0f, 0.0f) * Matrix.CreateTranslation(0, 0, zoom) * Matrix.CreateTranslation(1.5f, 0, -1.0f);
                boxEffect.Texture = boxTexture[1];
                boxEffect.Alpha = 1.0f;
                GraphicsDevice.DrawUserPrimitives<VertexPositionNormalTexture>(PrimitiveType.TriangleList,
                    box[1].Vertices, 0, box[1].Vertices.Length / 3);
            }
            // Draw quad1 & quad2

            foreach (EffectPass pass in quadEffect.CurrentTechnique.Passes)
            {
                pass.Apply();
                // Draw the quad1
                quadEffect.World = Matrix.CreateFromYawPitchRoll(0.0f, 0.0f, MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(0, 0, zoom*-0.2f);
                quadEffect.Texture = quadTexture[0];
                quadEffect.Alpha = 1.0f;
                GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionNormalTexture>(PrimitiveType.TriangleList,
                    quad[0].Vertices, 0, 4, quad[0].Indices, 0, 2);
 
                pass.Apply();
                // Draw the quad2
                quadEffect.World = Matrix.CreateFromYawPitchRoll(0.0f, 0.0f, MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(0, 0, zoom*-0.2f);
                quadEffect.Texture = quadTexture[1];
                quadEffect.Alpha = 0.5f;
                GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionNormalTexture>(PrimitiveType.TriangleList,
                    quad[1].Vertices, 0, 4, quad[1].Indices, 0, 2);
            }

            GraphicsDevice.BlendState = BlendState.Opaque;
            GraphicsDevice.DepthStencilState = DepthStencilState.Default;
            
            base.Draw(gameTime);
        }
    }
}
